// VERSION 001
import WPOClient from '@/wpo/v1.0.0/wpo-client'

export default class WPOAgentClient {
  constructor (wpoId) {
    // console.log('CALL WPOAgentClient constructor()')
    this.wpoId = wpoId
    this.isLinked = false

    this.onLinked = null // fuction ()
    this.onUnlinked = null // fuction ()
    this.onStdout = null // fuction (pid, data, time)
    this.onEmit = null // fuction (name, data, time)

    let wpoClient = new WPOClient(wpoId)
    this.wpoClient = wpoClient
    this.buildEventsHandler()
  }

  buildEventsHandler () {
    let wpoClient = this.wpoClient

    wpoClient.onLinked = () => {
      // console.log('EVENT ON Linked')
      this.isLinked = true
      if (this.onLinked) this.onLinked()
    }

    wpoClient.onUnlinked = () => {
      // console.log('EVENT ON Unlinked')
      this.isLinked = false
      if (this.onUnlinked) this.onUnlinked()
    }

    wpoClient.onError = (err) => {
      console.log('EVENT ON Error err = ', err)
    }

    wpoClient.onEmit = (name, data, time) => {
      // console.log('EVENT emit time, name, data = ', time, name, data)
      if (this.onEmit) this.onEmit(name, data, time)
    }

    wpoClient.onStdout = (pid, data, time) => {
      // console.log('EVENT stdout time, pid, data = ', time, pid, data)
      if (this.onStdout) this.onStdout(pid, data, time)
    }
  }

  connect () {
    let wpoClient = this.wpoClient
    if (!wpoClient) return

    wpoClient.connect()
  }

  comment (str) {
    let wpoClient = this.wpoClient
    if (!wpoClient) return

    wpoClient.comment(str)
  }

  disconnect () {
    let wpoClient = this.wpoClient
    if (!wpoClient) return

    wpoClient.disconnect()
  }

  stdin (pid, data) {
    let wpoClient = this.wpoClient
    if (!wpoClient) return

    wpoClient.stdin(pid, data)
  }

  async Pwd () {
    let wpoClient = this.wpoClient
    if (!wpoClient) return

    let currentDirectory = ''

    try {
      let resultStr = await wpoClient.call('pwd', '{}')
      // console.log('Pwd():resultStr = ', resultStr)
      let result = JSON.parse(resultStr)
      currentDirectory = result.currentDirectory
    } catch (e) {
      console.log('Pwd():Error:', e.message)
      throw e
    }

    return currentDirectory
  }

  async Lstate (path) {
    let wpoClient = this.wpoClient
    if (!wpoClient) return

    let state = {}

    try {
      let resultStr = await wpoClient.call('lstate', `{ "path" : "${path}" }`)
      // console.log('Lstate():resultStr = ', resultStr)
      let result = JSON.parse(resultStr)
      // console.log('result = ', result)
      state = result
    } catch (e) {
      console.log('Lstate():Error:', e.message)
      throw e
    }

    return state
  }

  async Dir (path) {
    let wpoClient = this.wpoClient
    if (!wpoClient) return

    let dirList = {}

    try {
      let resultStr = await wpoClient.call('dir', `{ "path" : "${path}" }`)
      // console.log('Dir():resultStr = ', resultStr)
      let result = JSON.parse(resultStr)
      // console.log('result = ', result)
      dirList = result
    } catch (e) {
      console.log('Dir():Error:', e.message)
      throw e
    }

    return dirList
  }

  async DirTree (path) {
    let wpoClient = this.wpoClient
    if (!wpoClient) return

    let dirTree = {}

    try {
      let resultStr = await wpoClient.call('dir:tree', `{ "path" : "${path}" }`)
      // console.log('Dir():resultStr = ', resultStr)
      let result = JSON.parse(resultStr)
      // console.log('result = ', result)
      dirTree = result
    } catch (e) {
      console.log('Dir():Error:', e.message)
      throw e
    }

    return dirTree
  }

  async Rename (fromPath, toPath) {
    let wpoClient = this.wpoClient
    if (!wpoClient) return

    let result = false

    try {
      await wpoClient.call('rename', `{ "from" : "${fromPath}", "to" : "${toPath}" }`)
      // let resultStr = await wpoClient.call('rename', `{ "from" : "${fromPath}", "to" : "${toPath}" }`)
      // console.log('Rename():resultStr = ', resultStr)
      // let result = JSON.parse(resultStr)
      // console.log('result = ', result)
      result = true
    } catch (e) {
      console.log('Rename():Error:', e.message)
      throw e
    }

    return result
  }

  async RunCommand (path, args) {
    let wpoClient = this.wpoClient
    if (!wpoClient) return

    let dataObj = { path, args }
    let data = JSON.stringify(dataObj)

    // console.log('RunCommand():data = ', data)
    let result = { runOutput: '' }

    try {
      let resultStr = await wpoClient.call('run', data)
      // console.log('RunCommand():resultStr = ', resultStr)
      result = JSON.parse(resultStr)
    } catch (e) {
      console.log('RunCommand():Error:', e.message)
      throw e
    }

    return result
  }

  async RunProcess (path, args) {
    let wpoClient = this.wpoClient
    if (!wpoClient) return

    let dataObj = { path, args }
    let data = JSON.stringify(dataObj)

    // console.log('RunProcess():data = ', data)
    let result = { runPid: '' }

    try {
      let resultStr = await wpoClient.call('process', data)
      // console.log('RunProcess():resultStr = ', resultStr)
      result = JSON.parse(resultStr)
    } catch (e) {
      console.log('RunProcess():Error:', e.message)
      throw e
    }

    return result
  }

  async RunPty (path, args, rows, cols, single) {
    let wpoClient = this.wpoClient
    if (!wpoClient) return

    let dataObj = { path, args, rows: String(rows), cols: String(cols) }
    let data = JSON.stringify(dataObj)

    console.log('RunPty():data = ', data)
    let result = { runPid: '' }

    try {
      let callname = 'pty'
      if (single) callname = callname + ':single'

      let resultStr = await wpoClient.call(callname, data)
      // console.log('RunPty():resultStr = ', resultStr)
      result = JSON.parse(resultStr)
    } catch (e) {
      console.log('RunPty():Error:', e.message)
      throw e
    }

    return result
  }

  async ReadFileAll (filePath) {
    let wpoClient = this.wpoClient
    if (!wpoClient) return

    let dataObj = { path: filePath }
    let data = JSON.stringify(dataObj)

    let result = { readData: '' }

    try {
      let resultStr = await wpoClient.call('read:file:all', data)
      // console.log('ReadFileAll():resultStr = ', resultStr)
      let resultObj = JSON.parse(resultStr)
      // console.log('resultObj = ', resultObj)
      switch (resultObj.type) {
        case 'base64' :
          result.readData = Buffer.from(resultObj.data, 'base64')
          break
        default : throw new Error('no support type on ReadFileAll()')
      }
    } catch (e) {
      console.log('ReadFileAll():Error:', e.message)
      throw e
    }

    return result
  }

  async WriteFileOnSyscall (filePath, fileData) {
    let wpoClient = this.wpoClient
    if (!wpoClient) return

    let dataObj = { path: filePath, type: 'utf8', data: fileData }

    switch (typeof fileData) {
      case 'string': dataObj.type = 'utf8'; break
      default: console.error('not support type of fileData type = ', typeof fileData); return
    }

    let data = JSON.stringify(dataObj)

    let result = { writeState: 'fail' }

    try {
      await wpoClient.call('write:file:syscall', data)
      // let resultStr = await wpoClient.call('write:file:syscall', data)
      // console.log('WriteFileOnSyscall():resultStr = ', resultStr)
      result.writeState = 'success'
    } catch (e) {
      console.log('WriteFileOnSyscall():Error:', e.message)
      throw e
    }

    return result
  }
}
