// import { delay } from '@/lib/utils-v0.0.1'
import { EventBus } from '@/event-bus'
import WPOClient from '@/lib/web-pass-on/wpo-agent-client-v1.9.0'

const state = {
  wpoID: '',
  lastWpoID: '',

  wpoClient: null,
  isLinked: false,

  valueInfo: [],
  checkValue: '1',
  checkValueUnit: '',
  readValue: '1',
  readValueUnit: '',
  isResultOk: false
}

const getters = {
  wpoID (_state) { return _state.wpoID },
  lastWpoID (_state) { return _state.lastWpoID },

  isLinked (_state) { return _state.isLinked },

  valueInfo (_state) { return _state.valueInfo },
  checkValue (_state) { return _state.checkValue },
  checkValueUnit (_state) { return _state.checkValueUnit },
  readValue (_state) { return _state.readValue },
  readValueUnit (_state) { return _state.readValueUnit },
  isResultOk (_state) { return _state.isResultOk }
}

const mutations = {
  wpoID (_state, payload) {
    _state.wpoID = payload
    if (payload) _state.lastWpoID = payload
  },
  isLinked (_state, payload) { _state.isLinked = payload },

  valueInfo (_state, payload) { _state.valueInfo = payload },
  checkValue (_state, payload) { _state.checkValue = payload },
  checkValueUnit (_state, payload) { _state.checkValueUnit = payload },
  readValue (_state, payload) { _state.readValue = payload },
  readValueUnit (_state, payload) { _state.readValueUnit = payload },
  isResultOk (_state, payload) { _state.isResultOk = payload }
}

const actions = {

  createWPOClient ({ state, dispatch, commit }, wpoID) {
    console.log('CALL json-single-json/createWPOClient() wpoID = ', wpoID)
    dispatch('closeWPOClient')

    commit('wpoID', wpoID)

    state.wpoClient = new WPOClient(wpoID)
    state.wpoClient.connect()
    state.wpoClient.comment('json single json client v1.0.0')

    let wpoClient = state.wpoClient
    wpoClient.onLinked = async () => {
      console.log('EVENT ON Linked')
      commit('isLinked', true)
      EventBus.$emit('json-single-json:linked')
    }

    wpoClient.onUnlinked = () => {
      console.log('EVENT ON Unlinked')
      commit('isLinked', false)
      EventBus.$emit('json-single-json:unlinked')
    }

    wpoClient.onStdout = (pid, data, time) => {
      console.log('EVENT ON Stdout data = ', data)
    }

    wpoClient.onEmit = (name, data, time) => {
      console.log('EVENT ON emit name, data = ', name, data)
    }
  },

  closeWPOClient ({ state, commit }) {
    console.log('CALL json-single-json/closeWPOClient()')
    if (!state.wpoClient) return
    let wpoClient = state.wpoClient
    wpoClient.disconnect()
    state.wpoClient = null
    commit('isLinked', false)
  },
  async getValueInfoFromProc ({ state, commit }) {
    console.log('CALL json-single-json/getValueInfoFromProc()')
    if (!state.wpoClient) return
    let wpoClient = state.wpoClient

    let infoArray = []
    let setNumber = 0
    let list = {}

    try {
      // 깃트 명령을 실행해서 버전을 얻는다.
      let runPath = 'cat'
      let runArgs = ['/proc/cpuinfo']
      let result = await wpoClient.RunCommand(runPath, runArgs)
      // console.log('result = ', result)
      let outputStr = result.runOutput
      // console.log('outputStr = ', outputStr)
      let outputStrings = outputStr.split('\n')
      // console.log('outputStrings = ', outputStrings)

      console.log('start setNumber = ', setNumber)
      console.log('start infoArray[setNumber] = ', infoArray)

      outputStrings.forEach(infoStr => {
        // console.log('infoStr  = ', infoStr)
        try {
          if (infoStr) {
            let fields = infoStr.split(':')
            // console.log('fields  = ', fields)
            let fieldName = fields[0].trim()
            let value = fields[1].trim()

            console.log('fieldName = ', fieldName)
            console.log('value = ', value)
            if (fieldName === 'processor') {
              // console.log('fieldName OK')
              setNumber = Number(value)
              list = {}
              if (setNumber === 0) {
                infoArray = []
              }
              // console.log('setNumber = ', setNumber)
            }

            let infoObj = { name: fieldName, value: value }
            list[fieldName] = infoObj
            infoArray[setNumber] = list
            console.log('list = ', list)
            console.log('setNumber = ', setNumber)
            console.log('infoArray[setNumber] = ', infoArray)
          }
        } catch (e) {
          console.log('getValueInfoFromProc() infoStr err  = ', e)
        }
      })
    } catch (e) {
      console.log('getValueInfoFromProc() err  = ', e)
    }
    console.log('valueInfo = ', infoArray)

    commit('valueInfo', infoArray)
    return infoArray
  },

  async kTesting  ({ state, commit, dispatch }) {
    console.log('CALL json-single-json/kTesting()')
    let ValueCounter = []
    if (!state.wpoClient) return

    let valueInfo = await dispatch('getValueInfoFromProc')
    console.log('valueInfo = ', valueInfo)
    console.log('valueInfo.length = ', valueInfo.length)
    if (valueInfo.length === 0) {
      commit('readValue', '0')
      commit('readValueUnit', '')
      return
    }

    valueInfo.forEach(element => {
      let checkData = true
      console.log('element = ', element)
      console.log(element['physical id'].value)
      if (ValueCounter.length === 0) {
        ValueCounter.push(element['physical id'].value)
      }
      ValueCounter.forEach(unit => {
        console.log(element['physical id'].value)
        console.log(unit)
        if (element['physical id'].value === unit) { checkData = false }
      })
      if (checkData) {
        ValueCounter.push(element['physical id'].value)
      }
    })

    commit('readValue', ValueCounter.length + '')
    commit('readValueUnit', '')

    let checkCounter = parseInt(state.checkValue)
    let readCounter = parseInt(ValueCounter.length)

    console.log('readCounter = ', readCounter)
    console.log('checkCounter = ', checkCounter)

    commit('isResultOk', readCounter === checkCounter)
  }
}

export default {
  namespaced: true,
  state,
  getters,
  mutations,
  actions
}
